package net.csn.dr2d.wscp;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.json.JsonObject;

import net.csn.dr2d.StgyCall;
import net.csn.dr2d.StgyReply;

/**
 * 
 * @author Llorenç Corbey @ strat-num.fr 
 *
 */
public class CnslPlayerAcknowledgement {

	CnslPlayerActions _CnslPlayerInstance = null;

	public CnslPlayerAcknowledgement(){	}
	protected void self(final CnslPlayerActions cnslPlayerInstance){
		_CnslPlayerInstance = cnslPlayerInstance;
	}
	
	public StgyReply acknowledge(final StgyCall stgyCall) throws IllegalStateException {

		final String seqName = stgyCall.getSequence().getValue();
		StgyReply stgyReply = StgyReply.acknowledge(stgyCall);

		if(_CnslPlayerInstance==null) throw new IllegalStateException("Missing CnslPlayerInstance!");

		try{
			final JsonObject argsIn  = stgyCall.getJSonArgsIn();
			final JsonObject argsRef = stgyCall.getJSonArgsRef();
			
			// Instantiate dynamically the related (to sequenceName) args class ...
			String argsClassName = String.format("%s.CnslPlayerArgs_%s", CnslPlayerAcknowledgement.class.getPackage().getName(), seqName); // ex: net.csn.dr2d.wscp.CnslPlayerArgs_InitPlayerFeatures
			Class<?> argsClass = Class.forName(argsClassName);
			Constructor<?> argsClassCtr = argsClass.getDeclaredConstructor(JsonObject.class, JsonObject.class);
			Object args = argsClassCtr.newInstance(argsIn, argsRef);
			// like in
			//final CnslPlayerArgs_InitPlayerFeatures args = new CnslPlayerArgs_InitPlayerFeatures(jsonData);
			
			// Invoke dynamically the related (to sequenceName) action ...
			String actionMethod = String.format("on%s", seqName);
			CnslPlayerActions.class.getMethod(actionMethod, argsClass).invoke(_CnslPlayerInstance, args);
			
			stgyReply.setJSonArgsRef((JsonObject)args.getClass().getMethod("toJsonObject").invoke(args));
		
		}catch(   ClassNotFoundException | InstantiationException
				| NoSuchMethodException 
				| IllegalArgumentException | IllegalAccessException | InvocationTargetException 
				ex)	{
			Logger.getLogger(CnslPlayerAcknowledgement.class.getName()).log(Level.INFO, "Crash!;)", ex); // During DEBUG
			java.lang.System.out.println("Crash!");
		}
		
		return stgyReply;
	}

}
