package net.csn.dr2d.wscp;

public interface CnslPlayerActions {
	
    /**
     * Premier appel du jeu.<br/> 
     * Fournit un Identifiant Unique de Session (IUS) qui permet de réentrer
     * dans le jeu aprés l'avoir quitté prématurément ... à garder si besoin ...<br/>
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : .playerUID 
	 * <li>Paramètres modifiables      : aucun
	 * </ul>
     */
	public void onGameInitialisation(final CnslPlayerArgs_GameInitialisation args);

	/**
	 * Second appel du jeu.<br/>
	 * Est appelé autant de fois qu'il y a de plateaux (traditionellement une fois).<br/>
	 * Reçoit la configuration du plateau identifié par boardIdx.<br/>
	 *  
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : .board {@see Board}
	 * <li>Paramètres modifiables      : aucun
	 * </ul>
	 * 
	 * L'objet 'board' contient l'index du plateau, ses dimmensions (communément 12x12),
	 * ainsi qu'une liste de définition des dalles actives, par exemple, laser en 2:3, presse en 4:3, etc<br/>
	 * NB: Les dalles non listées sont les dalles neutres
	 */
	public void onInitDeclareBoard(final CnslPlayerArgs_InitDeclareBoard args);

	/**
	 * Troisième appel du jeu.<br/>
	 * Le joueur peut personaliser son avatar.<br/> 
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : .mcndId
	 * <li>Paramètres modifiables      : .pseudo, .argbBackColor
	 * </ul>
	 * 
	 * Rq: Le pseudo est tronqué aprés 11 caractères. 
	 */
	public void onInitPlayerFeatures(final CnslPlayerArgs_InitPlayerFeatures args);
	
	/**
	 * Quatrième appel du jeu.<br/>
	 * Le joueur doit retourner la position initiale du mécanoïd et son orientation.<br/> 
	 * Si la position retournée est déja occupée ou invalide, cet appel sera  renouvellé (jusqu'à huit fois).<br/>
	 *
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <lu>
	 * <li>final ArrayList&lt;Integer&gt; <i>args.</i><b>availableTiles</b></li>
	 * </lu>
	 * <li>Paramètres modifiables      :
	 * <lu>
	 * <li>Position <i>args.</i><b>position</b></li>
	 * <li>Windrose <i>args.</i><b>azimuth</b></li>
	 * </lu>
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Position
	 * @see net.csn.dr2d.Windrose
	 */
	public void onInitPlayerLocalization(final CnslPlayerArgs_InitPlayerLocalization args);

	/**
	 * Cinquième appel du jeu.<br/>
	 * Reçoit la liste des participants (de 2 à 8 éléments).<br>
	 * Chaque participant est décrit par un identifiant, un avatar, sa position et son orientation de départ. 
	 *  
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <lu>
	 * <li>final Hashtable&lt;String, Competitor&gt; <i>args.</i><b>competitors</b></li>
	 * </lu>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Competitor
	 */
	public void onInitStatusOfCompetitors(final CnslPlayerArgs_InitStatusOfCompetitors args);
	
	/**
	 * Sixième appel du jeu.<br/>
	 * Reçoit la position des dalles sur lesquelles se trouvent chacun des artéfacts<br>
	 * ( les trois lois de la robotique qu'il faut atteindre pour terminer le jeu).
	 *  
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final ArrayList&lt;Position&gt; args.artefacts</li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Position
	 */
	public void onInitLocateArtefacts(final CnslPlayerArgs_InitLocateArtefacts args);
	
	/**
	 * Septième appel du jeu.<br/>
	 * Marque la fin des appels d'initialisations. C'est à ce moment que la partie débute ...
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : - aucun -
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 */
	public void onQuestStart(final CnslPlayerArgs_QuestStart args);
	
	/**
	 * Premier appel de boucle.<br>
	 * Le joueur doit retourner un cartouche composé de zéro à cinq mouvements qu'il souhaite voir se réaliser.<br> 
	 * Un mouvement est lui même composé de zero ou une translation accompagnée de zero ou une rotation.
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : - aucun -
	 * <li>Paramètres modifiables      :
	 * <ul>
	 * <li>Cartouche <i>args.</i><b>cartouche</b></li>
	 * </ul>
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Cartouche
	 */
	public void onRoundRequireCartouche(final CnslPlayerArgs_RoundRequireCartouche args);
	
	/**
	 * Second appel de boucle.<br>
	 * On reçoit le mouvement réellement réalisé par chacun des mécanoïds sous forme d'un tableau indexé sur l'identifiant.<br>
	 * Remarque1: Ces mouvements peuvent être différents de ceux prévus en cas de heurts, de nid de poule, de mur etc ...<br>
	 * Remarque2: L'index du tableau prend la forme d'une chaine "mcnd" suivie de l'identifiant, par exemple pour le mécanoïd n°2, l'index sera "mcnd02". 
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final Hashtable&lt;String, Movement&gt; <i>args.</i><b>allRobotsMovements</b></li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Movement
	 */
	public void onRoundRobotMovement(final CnslPlayerArgs_RoundRobotMovement args);

	/**
	 * Troisième et quatrième appel de boucle.<br>
	 * Reçoit, s'il y a lieu, le mouvement subit par chacun des mécanoïds en réponse aux événements du plateau (convoyeurs, pousseurs, etc...).
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final Hashtable&lt;String, Movement&gt; <i>args.</i><b>allBoardMovements</b></li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Movement
	 */
	public void onRoundBoardMovement(final CnslPlayerArgs_RoundBoardMovement args);

	/**
	 * Cinquième appel de boucle.<br>
	 * Reçoit la liste de l'ensemble des participants avec leur position, leur orientation et le nombre de points de capacité (PC (energie)) qu'ils leur reste.<br> 
	 * Remarque: L'index du tableau est l'identifiant court du mécanoïd concerné, par exemple pour le mécanoïd n°2, l'index sera "02".
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final Hashtable&lt;String, Competitor&gt; <i>args.</i><b>competitors</b></li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 * 
	 * @see net.csn.dr2d.Competitor
	 */
	public void onRoundStatusOfCompetitors(final CnslPlayerArgs_RoundStatusOfCompetitors args);
		
	/**
	 * Sixiéme appel de boucle.<br>
	 * Reçoit la définition de l'action qui s'est produite sur le plateau.<br>
	 * Remarque: Cet appel est facultatif. Par exemple, il est diffusé seulement lors des parties où la dalle action est activée.
	 *  
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final String <i>args.</i><b>actionName</b></li>
	 * <li>final String <i>args.</i><b>actionArgs</b></li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 */
	public void onRoundBoardAction(final CnslPlayerArgs_RoundBoardAction args);
	
	/**
	 * Dernier appel de boucle.<br>
	 * Cet appel, effectué quand nécéssaire, marque la fin de partie pour l'un ou l'ensemble des participants.<br>
	 * Les mécanoïds listés sont ceux ayant achevé la quête.<br>
	 * Remarque: Les valeurs listées représentent l'identifiant court du/des mécanoïd(s) concerné(s), par exemple pour le mécanoïd n°2, la valeur sera "02".
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule :
	 * <ul>
	 * <li>final ArrayList&lt;String&gt; <i>args.</i><b>questAchieved</b></li>
	 * </ul>
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 */
	public void onQuestAchieved(final CnslPlayerArgs_QuestAchieved args);

	/**
	 * Dernier appel du jeu.<br>
	 * Permet de libérer toutes les ressources utilisées. À partir de là, le serveur rompt la communication. 
	 * 
	 * @param args
	 * <ul>
	 * <li>Paramètres en lecture seule : - aucun -
	 * <li>Paramètres modifiables      : - aucun -
	 * </ul>
	 */
	public void onGameFinalisation(final CnslPlayerArgs_GameFinalisation args);
	
}
